package net.eusashead.hateoas.response.impl;

/*
 * #[license]
 * spring-responseentitybuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */


import java.util.Date;

import javax.servlet.http.HttpServletRequest;

import net.eusashead.hateoas.header.ETagHeaderStrategy;
import net.eusashead.hateoas.response.EntityResponseBuilder;

import org.springframework.http.ResponseEntity;

public class EntityResponseBuilderImpl<T> extends AbstractEntityResponseBuilder<T,T> implements EntityResponseBuilder<T> {

	public EntityResponseBuilderImpl(HttpServletRequest request) {
		super(request);
	}
	
	@Override
	protected void assertVerb() {
		if (!(this.request.getMethod().equals("GET") || this.request.getMethod().equals("HEAD"))) {
			throw new RuntimeException("This ResponseBuilder handles GET or HEAD requests only.");
		}
	};

	@Override
	public EntityResponseBuilder<T> entity(T entity) {
		super.setEntity(entity);
		return this;
	}

	@Override
	public EntityResponseBuilder<T> etag() {
		super.setEtagHeader();
		return this;
	}

	@Override
	public EntityResponseBuilder<T> etag(ETagHeaderStrategy strategy) {
		super.setEtagHeader(strategy);
		return this;
	}

	@Override
	public EntityResponseBuilder<T> etag(Date date) {
		super.setEtagHeader(date);
		return this;
	}

	@Override
	public EntityResponseBuilder<T> etag(Integer version) {
		super.setEtagHeader(version);
		return this;
	}
	
	@Override
	public EntityResponseBuilder<T> etag(Long version) {
		super.setEtagHeader(version);
		return this;
	}

	@Override
	public EntityResponseBuilder<T> lastModified(Date date) {
		super.setLastModifiedHeader(date);
		return this;
	}

	@Override
	public EntityResponseBuilder<T> expireIn(long millis) {
		super.setExpiryHeaders(millis);
		return this;
	}

	@Override
	public ResponseEntity<T> build() {
		super.assertEntity();
		
		return super.buildResponseEntity(this.entity);
		
	}

}
