package net.eusashead.hateoas.response.impl;

/*
 * #[license]
 * spring-responseentitybuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

import javax.servlet.http.HttpServletRequest;

import net.eusashead.hateoas.header.impl.AllowHeaderImpl;
import net.eusashead.hateoas.response.OptionsResponseBuilder;

import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

public class OptionsResponseBuilderImpl<T> extends AbstractEntityResponseBuilder<T,T> implements OptionsResponseBuilder<T> {
	
	public OptionsResponseBuilderImpl(HttpServletRequest request) {
		super(request);
	}
	
	@Override
	protected void assertVerb() {
		if (!this.request.getMethod().equals("OPTIONS")) {
			throw new RuntimeException("This ResponseBuilder handles OPTIONS requests only.");
		}
	};
	
	@Override
	public OptionsResponseBuilder<T> entity(T entity) {
		super.setEntity(entity);
		return this;
	}

	@Override
	public OptionsResponseBuilder<T> allow(HttpMethod... methods) {
		this.headers.setAllow(new AllowHeaderImpl(methods).getValue());
		return this;
	}
	
	@Override
	public ResponseEntity<T> build() {
		ResponseEntity<T> responseEntity;
		if (entity == null) {
			responseEntity = new ResponseEntity<T>(headers, HttpStatus.OK);
		} else {
			responseEntity = new ResponseEntity<T>(entity, headers, HttpStatus.OK);
		}
		return responseEntity;
	}

}
