package net.eusashead.hateoas.response.impl;

/*
 * #[license]
 * spring-responseentitybuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

import javax.servlet.http.HttpServletRequest;

import net.eusashead.hateoas.header.impl.LocationHeaderImpl;
import net.eusashead.hateoas.response.PostResponseBuilder;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

public class PostResponseBuilderImpl extends AbstractResponseBuilder<Void> implements PostResponseBuilder {

	protected final String domain;

	public PostResponseBuilderImpl(HttpServletRequest request) {
		super(request);

		// Set the root URL
		this.domain = request.getScheme() + "://" + request.getServerName();
	}
	
	@Override
	protected void assertVerb() {
		if (!this.request.getMethod().equals("POST")) {
			throw new RuntimeException("This ResponseBuilder handles POST requests only.");
		}
	};

	@Override
	public PostResponseBuilder location(String uri) {
		this.headers.setLocation(new LocationHeaderImpl(uri).getValue());
		return this;
	}

	@Override
	public PostResponseBuilder location(String uriTemplate, Object... objects) {
		this.headers.setLocation(new LocationHeaderImpl(uriTemplate, objects).getValue());
		return this;
	}

	@Override
	public ResponseEntity<Void> build() {
		if (this.headers.getLocation() == null) {
			throw new RuntimeException("Location header must be set before calling create().");
		}
		return new ResponseEntity<Void>(headers, HttpStatus.CREATED);
	}

}
